import numpy as np

from context import offfile
from context import sphere
from context import cells

def test_ismax():
    # open the test file
    vertices = np.genfromtxt("tests/data/eight_min.csv", delimiter=" ")

    # inputs for the test
    centre = np.array([0.294900, -0.075460, -1.194000])
    radius = 0
    basis = np.array([346])
    relative = np.array([316, 358, 317, 320, 352, 318, 321, 324, 356, 354,
        334, 373, 319, 325, 375, 329, 327, 378, 333, 370, 372, 377, 368, 323,
        332, 371, 330, 369])
    max_radius = 0.020000

    # check is max
    m = cells.ismax(vertices, centre, radius, basis, relative, max_radius)
    assert m == True

    # inputs for the test
    centre = np.array([0.425300, -0.075480, -1.099000])
    radius = 0
    basis = np.array([403])
    relative = np.array([320, 380, 357, 346, 381, 321, 408, 322, 367, 356,
        334, 407, 419, 335, 423, 424, 329, 358, 375, 331, 327, 326, 370, 378,
        374, 328, 379, 376])
    max_radius = 0.07

    # check is max
    m = cells.ismax(vertices, centre, radius, basis, relative, max_radius)
    assert m == False

def test_enumerate():
    # open the test file
    vertices = np.genfromtxt("tests/data/eight_enum.csv", delimiter=" ")

    # inputs for the test
    fixed = np.array([])
    others = np.array([316, 317, 318, 319, 320, 321, 323, 324, 325, 327, 329,
        330, 332, 333, 334, 345, 352, 353, 354, 356, 368, 369, 370, 371, 372,
        373, 375, 377, 378])
    explored = np.array([])
    epsilon = 0.01

    # check enumerate
    centre_list, radius_list, basis_list = cells.enumerate(vertices, fixed,
        others, explored, epsilon)

    assert centre_list.shape == (29, 3)
    assert centre_list[0][0] == -0.177500
    assert centre_list[0][1] == -0.075330
    assert centre_list[0][2] == -1.242000
    assert centre_list[28][0] == 0.03912
    assert centre_list[28][1] == 0.18340
    assert centre_list[28][2] == -1.04700

    assert basis_list.shape == (29, 3)
    assert (basis_list[0] == np.array([345, 0, 0])).all()
    assert (basis_list[1] == np.array([353, 0, 0])).all()
    assert (basis_list[2] == np.array([316, 0, 0])).all()
    assert (basis_list[3] == np.array([320, 0, 0])).all()
    assert (basis_list[4] == np.array([321, 0, 0])).all()
    assert (basis_list[5] == np.array([317, 0, 0])).all()
    assert (basis_list[6] == np.array([352, 0, 0])).all()
    assert (basis_list[7] == np.array([318, 0, 0])).all()
    assert (basis_list[8] == np.array([356, 0, 0])).all()
    assert (basis_list[9] == np.array([334, 0, 0])).all()
    assert (basis_list[10] == np.array([329, 0, 0])).all()
    assert (basis_list[11] == np.array([354, 0, 0])).all()
    assert (basis_list[12] == np.array([324, 0, 0])).all()
    assert (basis_list[13] == np.array([375, 0, 0])).all()
    assert (basis_list[14] == np.array([327, 0, 0])).all()
    assert (basis_list[15] == np.array([370, 0, 0])).all()
    assert (basis_list[16] == np.array([378, 0, 0])).all()
    assert (basis_list[17] == np.array([373, 0, 0])).all()
    assert (basis_list[18] == np.array([319, 0, 0])).all()
    assert (basis_list[19] == np.array([325, 0, 0])).all()
    assert (basis_list[20] == np.array([377, 0, 0])).all()
    assert (basis_list[21] == np.array([372, 0, 0])).all()
    assert (basis_list[22] == np.array([323, 0, 0])).all()
    assert (basis_list[23] == np.array([333, 0, 0])).all()
    assert (basis_list[24] == np.array([368, 0, 0])).all()
    assert (basis_list[25] == np.array([371, 0, 0])).all()
    assert (basis_list[26] == np.array([332, 0, 0])).all()
    assert (basis_list[27] == np.array([330, 0, 0])).all()
    assert (basis_list[28] == np.array([369, 0, 0])).all()

def test_recursive_enumerate():
    # open the test file
    vertices = np.genfromtxt("tests/data/eight_enum.csv", delimiter=" ")

    # create the index array
    index_array = np.empty([766])
    for i in range(766):
        index_array[i] = i

    centre_list, radius_list, basis_list = cells.recursive_enumerate(
        vertices, index_array, 0.01, 0)

    # open the text file to compare against
    expected = np.genfromtxt("tests/data/centre_recenum.txt", delimiter=" ")

    # compare against expected
    # equiv = True
    # for rc in np.nditer(centre_list):
    #     row_equiv = False
    #     for re in np.nditer(expected):
    #         if np.allclose(rc, re, atol=0.0001):
    #             row_equiv = True
    #             break
    #
    #     if not row_equiv:
    #         equiv = False
    #         break
    #
    # assert equiv == True
