import numpy as np
import numpy.linalg as la
import scipy.optimize as opt
import pytest

from context import entropy

def test_entropy():
    # load test files
    a = np.genfromtxt("tests/data/a.txt", delimiter=" ")
    p = np.genfromtxt("tests/data/p.txt", delimiter=" ")
    q_file = np.genfromtxt("tests/data/q.txt", delimiter=",")

    m, n = a.shape

    # copy of the closure to test
    def calculate_entropy(q):
        qda = np.dot(q.transpose(), a)
        qda_log = np.log2(qda)

        y = np.dot(-qda_log, p)

        ones = np.empty(n)
        ones.fill(1)

        adq = np.dot(a.transpose(), q)
        b = np.divide(ones, adq)

        dy = np.dot(np.dot(-a, np.diag(p)), b)

        return y, dy

    y_res, dy_res = calculate_entropy(q_file)

    # entropy is being returned correctly
    assert y_res == 9.9952711907309073

def test_q0():
    # load the test file
    q0 = np.genfromtxt("tests/data/q0.txt", delimiter=" ")
    res = np.divide(q0, la.norm(q0, 1))

def test_linprog():
    # load the test files
    ap = np.genfromtxt("tests/data/ap.txt", delimiter=",")
    b = np.genfromtxt("tests/data/bl.txt", delimiter=",")
    c = np.genfromtxt("tests/data/cl.txt", delimiter=",")

    lin_result = opt.linprog(c, np.empty(ap.shape), np.empty(ap.shape[0]),
        ap, b, (0, 1), options={'tol': 1e-06})

    print(lin_result.x)

def test_simplical_complex_entropy():
    # load test files
    a = np.genfromtxt("tests/data/a.txt", delimiter=" ")
    p = np.genfromtxt("tests/data/p.txt", delimiter=" ")

    e, err, q = entropy.simplical_complex_entropy(a, p)

    print("\ne:{}".format(e))
    print("err:{}".format(err))
    print("q:{}".format(q))
