import numpy as np
import numpy.linalg as la

import topology.cech.cells as cells
import topology.util.math as mutil
import topology.entropy.entropy as entropy
import topology.bounding.sphere as sphere

def calculate(vertices, epsilon):
    d = vertices.shape[1]
    n = vertices.shape[0]

    # recursively enumerate
    centre_list, radius_list, basis_list = cells.recursive_enumerate(
        vertices, np.arange(n), epsilon, 0)

    while True:
        m = basis_list.shape[0]
        a = np.zeros((m, n))

        for i in range(0, m):
            for j in range(0, n):
                a[i, j] = 1 if (la.norm(centre_list[i,:] - vertices[j,:])
                    <= radius_list[i] + mutil.epsilon) else 0

        p = np.ones(n) / n
        e, err, q = entropy.simplical_complex_entropy(a, p)

        max_r_i = np.argmax(radius_list)
        max_r = radius_list[max_r_i]

        if max_r < mutil.epsilon:
            break   # tested to here

        cb = basis_list[i,np.nonzero(basis_list[i,:] > 0)]
        v = vertices - np.dot(centre_list[i,:], np.ones((n, 1)))
        dist = np.sqrt(np.sum(v**2, axis=1))
        intersect = np.nonzero(dist < R * (1-mutil.epsilon))

        new_centre_list = np.empty([0, 3])
        new_radius_list = np.empty([0])
        new_basis_list = np.empty([0])

        for j in range(0, cb.shape.shape[0]):
            ip = np.r_[cb[j-1,0], cb[cb.shape[0]-1,j+1], intersect]

            c, r, b = sphere.minsphere(np.empty([0, 3]), ip[0],
                ip[1:ip.shape[0]-1], np.inf)

            v = vertices - np.dot(c, np.ones([n, 1]))
            dist = np.sqrt(np.sum(v**2, axis=1))
            intnew = np.nonzero(dist < r * (1 - mutil.epsilon))

            if (cells.ismax(c, r, b, np.arrange(n), max_r) and
                np.setdiff1d(intnew, ip).shape[0] == 0):
                new_basis_list = np.r_[new_basis_list, np.r_[b, np.zeros([1, d+1-b.shape[0]-1])].transpose()]
                new_centre_list = np.r_[new_centre_list, c]
                new_radius_list =np.r_[new_radius_list, r]

        m = basis_list.shape[1]
        basis_list = np.r_[basis_list[0:max_r_i-1,:], basis_list[max_r_i+1:m-1], new_basis_list]
        centre_list = np.r_[centre[0:max_r_i-1,:], centre[max_r_i+1:m-1], new_centre_list]
        radius_list = np.r_[radius_list[0:max_r_i-1], radius_list[max_r_i+1:m-1], new_radius_list]

        count+=1

    return centre_list, radius_list, basis_list
